@@ -1,5 +1,6 @@
 test_that("TerminatorNone works", {
   terminators = trms(c("evals", "none"))
+
   terminators[[1]]$param_set$values$n_evals = 10L
   expect_output(print(terminators[[2]]), "TerminatorNone")
   terminator = TerminatorCombo$new(terminators)
@@ -13,3 +14,8 @@ test_that("TerminatorNone works with empty archive", {
   archive = Archive$new(ps(x = p_dbl()), ps(y = p_dbl(tags = "minimize")))
   expect_false(terminator$is_terminated(archive))
 })
+
+test_that("man exists", {
+  terminator = trm("none")
+  expect_man_exists(terminator$man)
+})
